# -*- coding: utf-8 -*-
## Used to install ExternalSiteCatalog content
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module is used by portal_quickinstaller to install product
"""

__docformat__ = 'restructuredtext'

# Python imports
from StringIO import StringIO

# CMF
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import permissions as CCP

# Archetypes
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.public import listTypes

# Products import
from Products.ExternalSiteCatalog.config import PROJECTNAME
from Products.ExternalSiteCatalog.config import GLOBALS
from Products.ExternalSiteCatalog.config import TOOLNAME
from Products.ExternalSiteCatalog.config import zope_DEPS
from Products.ExternalSiteCatalog.config import plone_DEPS
from Products.ExternalSiteCatalog.tool import ExternalSiteCatalog
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod
from Products.PloneSubscription.installers.utils import InstallationRunner
from Products.PloneSubscription.installers.utils import InstallationContext
from Products.PloneSubscription.installers.RoleInstaller import RoleInstaller
from Products.PloneSubscription.installers.ActionsInstaller import ActionsInstaller
from Products.PloneSubscription.installers.StandardPropertiesInstaller import StandardPropertiesInstaller
from Products.PloneSubscription.installers.ToolInstaller import ToolInstaller
from Products.PloneSubscription.installers.SkinLayersInstaller import SkinLayersInstaller
from Products.PloneSubscription.installers.ConfigletsInstaller import ConfigletsInstaller
from Products.PloneSubscription.installers.PortletInstaller import PortletInstaller
from Products.PloneSubscription.installers.WorkflowInstaller import WorkflowInstaller
from Products.PloneSubscription.installers.CatalogInstaller import CatalogInstaller

# Configlets
externalsitecatalog_management_configlet = {
    'id': 'externalsitecatalog_management',
    'appId': PROJECTNAME,
    'name': 'External Site Catalog',
    'action': 'string:$portal_url/prefs_externalsitecatalog',
    'category': 'Products',
    'permission': (CCP.ManagePortal,),
    'imageUrl': 'externalsitecatalog_icon.gif',
    }
    
def getRunners(uninstall=0):

    installers = []

    pf_navtree_props =(
        ('metaTypesNotToList',
         ('ExternalSiteCatalog',
          'ExternalSiteCatalogTask',
          )
         ),
        )
    spi = StandardPropertiesInstaller('navtree_properties', pf_navtree_props)
    installers.append(spi)    
    
    sti = ToolInstaller(ExternalSiteCatalog)
    installers.append(sti)
    
    si = SkinLayersInstaller()
    installers.append(si)
    
    ci = ConfigletsInstaller(externalsitecatalog_management_configlet)
    installers.append(ci)
        
    pi = PortletInstaller(('here/external_catalog_search_form_portlet/macros/portlet',),
                          slot_prop_name='right_slots')
    installers.append(pi)     
    
    return InstallationRunner(*tuple(installers))

def install_archetypes_catalogs(self, out):
    """Install Archetypes Catalogs"""
    # We don't want external catalog specific content to appear in the portal_catalog.
    at=getToolByName(self, 'archetype_tool')
    for meta_type in ("ExternalSiteCatalogTask",
                      "ExternalSiteCatalog",
                      ):
        at.setCatalogsByType(meta_type,())

def install(self, reinstall=0):
    """Install ExternalSiteCatalog product"""
    
    out = StringIO()    
    
    # Install types
    typeInfo = listTypes(PROJECTNAME)
    installTypes(self, out,
                 typeInfo,
                 PROJECTNAME)

    install_archetypes_catalogs(self, out)

    # Always start with the creation of the InstallationContext
    ic = InstallationContext(self, GLOBALS)
    
    for product in zope_DEPS:
        ic.requiresProduct(product)
    for product in plone_DEPS:
        ic.requiresInstalledProduct(product)

    # Runs the installation and return the report
    out.write(getRunners().install(ic, auto_reorder=True))
        
    mtool = getToolByName(self, 'portal_maintenance')
    scriptsholder = mtool.scripts

    if not 'crawlExternalSite' in scriptsholder.objectIds():
        # Plone Catalog update script
        manage_addExternalMethod(scriptsholder,'crawlExternalSite',
            'Crawl external sites',
            PROJECTNAME+'.MaintenanceToolbox',
            'crawlExternalSite')

    out.write('Installation completed.\n')
    
    return out.getvalue()

def uninstall(self):

    # Always start with the creation of the InstallationContext
    ic = InstallationContext(self, GLOBALS)

    # Runs the uninstallation and return the report
    report = getRunners(uninstall=1).uninstall(ic, auto_reorder=True)

    return report
    
