# -*- coding: utf-8 -*-
## ExternalSiteCatalog
## Index and search external sites in a Plone site
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
"""

__author__ = 'Maik Rder <maik.roeder@ingeniweb.com>'
__docformat__ = 'restructuredtext'

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from DateTime import DateTime

from Testing import ZopeTestCase # side effect import. leave it here.
from Products.ExternalSiteCatalog.tests import externalsitecatalogtestcase
from Products.PloneMaintenance.PloneMaintenanceTool import getNoSecDate

tests = []

class TestSiteExternalCatalog(externalsitecatalogtestcase.ExternalSiteCatalogTestCase):
    def test_1_addsMaintenanceTask(self):
        self.failUnless(self.portal.portal_maintenance.tasks.objectIds()==[])
        request = {'taskId':None,
                   'title':"Task1",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':None,
                   'url':None}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failUnless(self.portal.portal_maintenance.tasks.objectIds()==['externalsitecatalogtask',])

    def test_2_get_sites_info(self):
        self.failUnless(self.portal.portal_externalcatalog.getExternalSitesInfo()==[])
        request = {'taskId':None,
                   'title':"Task1",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':None,
                   'url':"http://localhost:8080"}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failIf(self.portal.portal_externalcatalog.getExternalSitesInfo()==[])
        infos= self.portal.portal_externalcatalog.getExternalSitesInfo()
        self.failUnless(infos[0]['last_execution_date']==request['last_execution_date'])
        self.failUnless(infos[0]['title']==request['title'])

    def test_3_edit_site_neutrality(self):
        request = {'taskId':None,
                   'title':"Task1",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':None,
                   'url':"http://localhost:8080"}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        infos1= self.portal.portal_externalcatalog.getExternalSitesInfo()
        request.update({'taskId':'externalsitecatalogtask'})
        self.portal.portal_externalcatalog.editExternalSiteCatalogTask(request)
        infos2= self.portal.portal_externalcatalog.getExternalSitesInfo()
        self.failUnless(infos1==infos2)

    def test_4_edit_site_changes_title(self):
        request = {'taskId':None,
                   'title':"Task1",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':None,
                   'url':"http://localhost:8080"}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        infos1= self.portal.portal_externalcatalog.getExternalSitesInfo()
        request.update({'taskId':'externalsitecatalogtask',
                        'title':"Task2"})
        self.portal.portal_externalcatalog.editExternalSiteCatalogTask(request)
        infos2= self.portal.portal_externalcatalog.getExternalSitesInfo()
        self.failUnless(infos1!=infos2)

    def test_5_delete_site(self):
        request = {'taskId':None,
                   'title':"Task1",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':None,
                   'url':"http://localhost:8080"}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failUnless(self.portal.portal_externalcatalog.getExternalSitesInfo()!=[])
        self.portal.portal_externalcatalog.deleteExternalSiteCatalogTasks(['externalsitecatalogtask',])
        self.failUnless(self.portal.portal_externalcatalog.getExternalSitesInfo()==[])

    def test_6_get_task(self):
        request = {'taskId':None,
                   'title':"Task1",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':None,
                   'url':"http://localhost:8080"}
        attribute_error=0
        try:
            task = self.portal.portal_externalcatalog.getExternalSiteCatalogTask('externalsitecatalogtask')
        except AttributeError:
            attribute_error=1
        self.failUnless(attribute_error)
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        task = self.portal.portal_externalcatalog.getExternalSiteCatalogTask('externalsitecatalogtask')
        self.failUnless(task.title_or_id()=="Task1")

    def test_7_checking_for_task(self):
        self.failIf(self.portal.portal_externalcatalog.hasExternalSiteCatalogTask('externalsitecatalogtask'))
        request = {'taskId':None,
                   'title':"Task1",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':None,
                   'url':"http://localhost:8080"}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failUnless(self.portal.portal_externalcatalog.hasExternalSiteCatalogTask('externalsitecatalogtask'))

    def test_8_propertysheet(self):
        ps = self.portal.portal_externalcatalog.getPropertySheet(task_id=None)
        self.failUnless(ps.getTaskId()=="")
        self.failUnless(ps.getUrl()=="http://")
        self.failUnless(ps.getPeriod()=="daily")
        self.failUnless(ps.getScheduled_hour()==0)
        self.failUnless(ps.getMax_duration()==1)
        self.failUnless(ps.getMax_pages()==1)
        self.failUnless(ps.getLast_execution_date().year()==DateTime().year())
        self.failUnless(ps.getLogin()=="")
        self.failUnless(ps.getPassword()=="")
        self.failIf(ps.Title()=="Portal", "PropertySheet needs to implement Title method, otherwise the portal title is taken.")
        self.failUnless(ps.Title()=="")

    def test_9_propertysheet_schema(self):
        props = self.portal.portal_externalcatalog.getPropertySheet(task_id=None)
        schematas = props.Schemata()
        fieldsets_ = [key for key in schematas.keys() if key != 'metadata']
        self.failUnless(fieldsets_==['default'])
        self.failUnless([fs for fs in fieldsets_ if schematas[fs].filterFields()]==['default'])
        self.failUnless(len(schematas['default'].filterFields())==11)
        self.failUnless(props['title']=='')
        self.failUnless(props.title_or_id()=='')

    def test_10_maintenancescript(self):
        self.failUnless(self.portal.portal_maintenance.tasks.objectIds()==[])
        self.failUnless('crawlExternalSite' in self.portal.portal_maintenance.scripts.objectIds())
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':'daily',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.portal.portal_maintenance.runTasks()
        self.failUnless("crawlExternalSite" in self.portal.portal_maintenance.scripts.objectIds())
        task = self.portal.portal_maintenance.tasks.externalsitecatalogtask
        self.failUnless(task.script_name=="crawlExternalSite")

    def test_11_crawling(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime()-10,
                   'max_duration':24,
                   'max_pages':1,
                   'scheduled_hour':0,
                   'period':'daily',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failIf(self.portal.portal_maintenance.isRunning())

        task = self.portal.portal_maintenance.tasks.externalsitecatalogtask
        date = getNoSecDate(DateTime())
        self.failUnless(date)
        self.portal.portal_maintenance.runTasks()
        self.failUnless(len(self.portal.portal_externalcatalog())>0)

    def test_12_daily_task_schedule(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':'daily',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        for task in self.portal.portal_maintenance.tasks.objectValues():
            self.failUnless(task.scheduled_day_of_month == '*')
            self.failUnless(task.scheduled_hour == '1')
            self.failUnless(task.scheduled_month == '*')
            self.failUnless(task.scheduled_minute == '*')

    def test_13_monthly_task_schedule(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime(),
                   'max_duration':1,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':'monthly',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        for task in self.portal.portal_maintenance.tasks.objectValues():
            self.failUnless(task.scheduled_day_of_month == '1')
            self.failUnless(task.scheduled_hour == '1')
            self.failUnless(task.scheduled_month == '*')
            self.failUnless(task.scheduled_minute == '*')

    def test_14_indexExternalSiteCatalogTasks(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime()-10,
                   'max_duration':24,
                   'max_pages':1,
                   'scheduled_hour':1,
                   'period':'daily',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failIf(self.portal.portal_maintenance.isRunning())
        self.portal.portal_externalcatalog.indexExternalSiteCatalogTasks(ids=['externalsitecatalogtask'])
        self.failUnless(len(self.portal.portal_externalcatalog())>0)

    def test_15_unindex_deleted_sites(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime()-10,
                   'max_duration':24,
                   'max_pages':10,
                   'scheduled_hour':1,
                   'period':'daily',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.portal.portal_externalcatalog.indexExternalSiteCatalogTasks(ids=['externalsitecatalogtask'])
        self.failUnless(len(self.portal.portal_externalcatalog())>0)
        self.portal.portal_externalcatalog.deleteExternalSiteCatalogTasks(['externalsitecatalogtask',])
        self.failUnless(len(self.portal.portal_externalcatalog())==0)

    def test_16_indexExternalSiteCatalogTasks(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime()-10,
                   'max_duration':24,
                   'max_pages':0,
                   'scheduled_hour':1,
                   'period':'daily',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failIf(self.portal.portal_maintenance.isRunning())
        self.portal.portal_externalcatalog.indexExternalSiteCatalogTasks(ids=['externalsitecatalogtask'])
        self.failUnless(len(self.portal.portal_externalcatalog())==0)

    def test_17_indexExternalSiteCatalogTasks(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime()-10,
                   'max_duration':24,
                   'max_pages':1,
                   'scheduled_hour':1,
                   'period':'daily',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failIf(self.portal.portal_maintenance.isRunning())
        self.portal.portal_externalcatalog.indexExternalSiteCatalogTasks(ids=['externalsitecatalogtask'])
        self.failUnless(len(self.portal.portal_externalcatalog())==1)

    def test_18_indexExternalSiteCatalogTasks(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost",
                   'charset':'utf-8',
                   'login':None,
                   'password':None,
                   'last_execution_date':DateTime()-10,
                   'max_duration':24,
                   'max_pages':2,
                   'scheduled_hour':1,
                   'period':'daily',
                   'url':'http://localhost:8080'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        self.failIf(self.portal.portal_maintenance.isRunning())
        self.portal.portal_externalcatalog.indexExternalSiteCatalogTasks(ids=['externalsitecatalogtask'])
        self.failUnless(len(self.portal.portal_externalcatalog())==2)
        
    def test_19_in_duration(self):
        in_duration = self.portal.portal_externalcatalog.in_duration
        date = DateTime('2006/3/9 1:45pm') 
        self.failUnless(in_duration(date, scheduled_hour=12, duration=2))
        self.failUnless(in_duration(date, scheduled_hour=13, duration=1))
        self.failIf(in_duration(date, scheduled_hour=11, duration=1))
        self.failIf(in_duration(date, scheduled_hour=12, duration=0))
        self.failIf(in_duration(date, scheduled_hour=13, duration=0))
        self.failIf(in_duration(date, scheduled_hour=14, duration=0))

    def test_20_in_duration(self):
        in_duration = self.portal.portal_externalcatalog.in_duration
        date = DateTime('2006/3/9 23:45pm') 
        self.failUnless(in_duration(date, scheduled_hour=12, duration=14))
        self.failUnless(in_duration(date, scheduled_hour=23, duration=10))
        self.failUnless(in_duration(date, scheduled_hour=23, duration=1))
        self.failUnless(in_duration(date, scheduled_hour=24, duration=1))
        self.failIf(in_duration(date, scheduled_hour=1, duration=1))

    def test_21_in_duration(self):
        in_duration = self.portal.portal_externalcatalog.in_duration
        date = DateTime('2006/3/9 00:45pm') 
        self.failUnless(in_duration(date, scheduled_hour=12, duration=14))
        self.failUnless(in_duration(date, scheduled_hour=23, duration=10))
        self.failIf(in_duration(date, scheduled_hour=0, duration=1))
        self.failIf(in_duration(date, scheduled_hour=1, duration=1))

    def test_22_in_duration(self):
        in_duration = self.portal.portal_externalcatalog.in_duration
        date = DateTime('2006/5/30 17:28pm') 
        self.failIf(in_duration(date, scheduled_hour=16, duration=1))

    def test_23_basicauth(self):
        self.failUnless(len(self.portal.portal_externalcatalog())==0)
        request = {'taskId':None,
                   'title':"Localhost Manage",
                   'charset':'utf-8',
                   'login':'login',
                   'password':'password',
                   'last_execution_date':DateTime()-10,
                   'max_duration':24,
                   'max_pages':2,
                   'scheduled_hour':1,
                   'period':'daily',
                   'url':'http://localhost:8080/manage'}
        self.portal.portal_externalcatalog.addExternalSiteCatalogTask(request)
        task = self.portal.portal_maintenance.tasks.externalsitecatalogtask
        self.failUnless(task.getPassword()=='password')
        self.failUnless(task.getLogin()=='login')
        self.failIf(self.portal.portal_maintenance.isRunning())
        self.portal.portal_externalcatalog.indexExternalSiteCatalogTasks(ids=['externalsitecatalogtask'])
        self.failUnless(len(self.portal.portal_externalcatalog())==2)
        
tests.append(TestSiteExternalCatalog)

if __name__ == '__main__':
    framework()
else:
    # While framework.py provides its own test_suite()
    # method the testrunner utility does not.
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        for test in tests:
            suite.addTest(unittest.makeSuite(test))
        return suite
